from pypanther.rules.zoom_operation.zoom_all_meetings_secured_with_one_option_disabled import (
    ZoomAllMeetingsSecuredWithOneOptionDisabled as ZoomAllMeetingsSecuredWithOneOptionDisabled,
)
from pypanther.rules.zoom_operation.zoom_automatic_sign_out_disabled import (
    ZoomAutomaticSignOutDisabled as ZoomAutomaticSignOutDisabled,
)
from pypanther.rules.zoom_operation.zoom_new_meeting_passcode_required_disabled import (
    ZoomNewMeetingPasscodeRequiredDisabled as ZoomNewMeetingPasscodeRequiredDisabled,
)
from pypanther.rules.zoom_operation.zoom_operation_passcode_disabled import ZoomPasscodeDisabled as ZoomPasscodeDisabled
from pypanther.rules.zoom_operation.zoom_sign_in_method_modified import (
    ZoomSignInMethodModified as ZoomSignInMethodModified,
)
from pypanther.rules.zoom_operation.zoom_sign_in_requirements_changed import (
    ZoomSignInRequirementsChanged as ZoomSignInRequirementsChanged,
)
from pypanther.rules.zoom_operation.zoom_two_factor_authentication_disabled import (
    ZoomTwoFactorAuthenticationDisabled as ZoomTwoFactorAuthenticationDisabled,
)
from pypanther.rules.zoom_operation.zoom_user_promoted_to_privileged_role import (
    ZoomUserPromotedtoPrivilegedRole as ZoomUserPromotedtoPrivilegedRole,
)
