from pypanther.rules.netskope.netskope_admin_logged_out import (
    NetskopeAdminLoggedOutLoginFailures as NetskopeAdminLoggedOutLoginFailures,
)
from pypanther.rules.netskope.netskope_admin_user_change import NetskopeAdminUserChange as NetskopeAdminUserChange
from pypanther.rules.netskope.netskope_many_deletes import NetskopeManyDeletes as NetskopeManyDeletes
from pypanther.rules.netskope.netskope_personnel_action import (
    NetskopeNetskopePersonnelActivity as NetskopeNetskopePersonnelActivity,
)
from pypanther.rules.netskope.netskope_unauthorized_api_calls import (
    NetskopeUnauthorizedAPICalls as NetskopeUnauthorizedAPICalls,
)
