from pypanther.rules.crowdstrike.crowdstrike_base64_encoded_args import (
    CrowdstrikeBase64EncodedArgs as CrowdstrikeBase64EncodedArgs,
)
from pypanther.rules.crowdstrike.crowdstrike_connection_to_embargoed_country import (
    ConnectiontoEmbargoedCountry as ConnectiontoEmbargoedCountry,
)
from pypanther.rules.crowdstrike.crowdstrike_credential_dumping_tool import (
    CrowdstrikeCredentialDumpingTool as CrowdstrikeCredentialDumpingTool,
)
from pypanther.rules.crowdstrike.crowdstrike_cryptomining_tools import (
    CrowdstrikeCryptominingTools as CrowdstrikeCryptominingTools,
)
from pypanther.rules.crowdstrike.crowdstrike_detection_passthrough import (
    CrowdstrikeDetectionpassthrough as CrowdstrikeDetectionpassthrough,
)
from pypanther.rules.crowdstrike.crowdstrike_dns_request import CrowdstrikeDNSRequest as CrowdstrikeDNSRequest
from pypanther.rules.crowdstrike.crowdstrike_lolbas import CrowdstrikeFDRLOLBAS as CrowdstrikeFDRLOLBAS
from pypanther.rules.crowdstrike.crowdstrike_macos_add_trusted_cert import (
    CrowdstrikeMacosAddTrustedCert as CrowdstrikeMacosAddTrustedCert,
)
from pypanther.rules.crowdstrike.crowdstrike_macos_osascript_administrator import (
    CrowdstrikeMacosOsascriptAdministrator as CrowdstrikeMacosOsascriptAdministrator,
)
from pypanther.rules.crowdstrike.crowdstrike_macos_plutil_usage import (
    CrowdstrikeMacosPlutilUsage as CrowdstrikeMacosPlutilUsage,
)
from pypanther.rules.crowdstrike.crowdstrike_real_time_response_session import (
    CrowdstrikeRealTimeResponseSession as CrowdstrikeRealTimeResponseSession,
)
from pypanther.rules.crowdstrike.crowdstrike_remote_access_tool_execution import (
    CrowdstrikeRemoteAccessToolExecution as CrowdstrikeRemoteAccessToolExecution,
)
from pypanther.rules.crowdstrike.crowdstrike_reverse_shell_tool_executed import (
    CrowdstrikeReverseShellToolExecuted as CrowdstrikeReverseShellToolExecuted,
)
from pypanther.rules.crowdstrike.crowdstrike_systemlog_tampering import (
    CrowdstrikeSystemlogTampering as CrowdstrikeSystemlogTampering,
)
from pypanther.rules.crowdstrike.crowdstrike_unusual_parent_child_processes import (
    CrowdstrikeUnusualParentChildProcesses as CrowdstrikeUnusualParentChildProcesses,
)
from pypanther.rules.crowdstrike.crowdstrike_wmi_query_detection import (
    CrowdstrikeWMIQueryDetection as CrowdstrikeWMIQueryDetection,
)
