from pypanther.rules.microsoft.microsoft365_brute_force_login_by_user import (
    Microsoft365BruteForceLoginbyUser as Microsoft365BruteForceLoginbyUser,
)
from pypanther.rules.microsoft.microsoft365_external_sharing import (
    Microsoft365ExternalDocumentSharing as Microsoft365ExternalDocumentSharing,
)
from pypanther.rules.microsoft.microsoft365_mfa_disabled import Microsoft365MFADisabled as Microsoft365MFADisabled
from pypanther.rules.microsoft.microsoft_exchange_external_forwarding import (
    Microsoft365ExchangeExternalForwarding as Microsoft365ExchangeExternalForwarding,
)
from pypanther.rules.microsoft.microsoft_graph_passthrough import MicrosoftGraphPassthrough as MicrosoftGraphPassthrough
