from pypanther.rules.slack.slack_app_access_expanded import (
    SlackAuditLogsAppAccessExpanded as SlackAuditLogsAppAccessExpanded,
)
from pypanther.rules.slack.slack_app_added import SlackAuditLogsAppAdded as SlackAuditLogsAppAdded
from pypanther.rules.slack.slack_app_removed import SlackAuditLogsAppRemoved as SlackAuditLogsAppRemoved
from pypanther.rules.slack.slack_application_dos import SlackAuditLogsApplicationDoS as SlackAuditLogsApplicationDoS
from pypanther.rules.slack.slack_dlp_modified import SlackAuditLogsDLPModified as SlackAuditLogsDLPModified
from pypanther.rules.slack.slack_ekm_config_changed import (
    SlackAuditLogsEKMConfigChanged as SlackAuditLogsEKMConfigChanged,
)
from pypanther.rules.slack.slack_ekm_slackbot_unenrolled import (
    SlackAuditLogsEKMSlackbotUnenrolled as SlackAuditLogsEKMSlackbotUnenrolled,
)
from pypanther.rules.slack.slack_ekm_unenrolled import SlackAuditLogsEKMUnenrolled as SlackAuditLogsEKMUnenrolled
from pypanther.rules.slack.slack_idp_configuration_change import (
    SlackAuditLogsIDPConfigurationChanged as SlackAuditLogsIDPConfigurationChanged,
)
from pypanther.rules.slack.slack_information_barrier_modified import (
    SlackAuditLogsInformationBarrierModified as SlackAuditLogsInformationBarrierModified,
)
from pypanther.rules.slack.slack_intune_mdm_disabled import (
    SlackAuditLogsIntuneMDMDisabled as SlackAuditLogsIntuneMDMDisabled,
)
from pypanther.rules.slack.slack_legal_hold_policy_modified import (
    SlackAuditLogsLegalHoldPolicyModified as SlackAuditLogsLegalHoldPolicyModified,
)
from pypanther.rules.slack.slack_mfa_settings_changed import (
    SlackAuditLogsMFASettingsChanged as SlackAuditLogsMFASettingsChanged,
)
from pypanther.rules.slack.slack_org_created import SlackAuditLogsOrgCreated as SlackAuditLogsOrgCreated
from pypanther.rules.slack.slack_org_deleted import SlackAuditLogsOrgDeleted as SlackAuditLogsOrgDeleted
from pypanther.rules.slack.slack_passthrough_anomaly import (
    SlackAuditLogsPassthroughAnomaly as SlackAuditLogsPassthroughAnomaly,
)
from pypanther.rules.slack.slack_potentially_malicious_file_shared import (
    SlackAuditLogsPotentiallyMaliciousFileShared as SlackAuditLogsPotentiallyMaliciousFileShared,
)
from pypanther.rules.slack.slack_private_channel_made_public import (
    SlackAuditLogsPrivateChannelMadePublic as SlackAuditLogsPrivateChannelMadePublic,
)
from pypanther.rules.slack.slack_privilege_changed_to_user import (
    SlackAuditLogsUserPrivilegeChangedToUser as SlackAuditLogsUserPrivilegeChangedToUser,
)
from pypanther.rules.slack.slack_service_owner_transferred import (
    SlackAuditLogsServiceOwnerTransferred as SlackAuditLogsServiceOwnerTransferred,
)
from pypanther.rules.slack.slack_sso_settings_changed import (
    SlackAuditLogsSSOSettingsChanged as SlackAuditLogsSSOSettingsChanged,
)
from pypanther.rules.slack.slack_user_privilege_escalation import (
    SlackAuditLogsUserPrivilegeEscalation as SlackAuditLogsUserPrivilegeEscalation,
)
