from pypanther.rules.cisco_umbrella_dns.domain_blocked import CiscoUmbrellaDNSBlocked as CiscoUmbrellaDNSBlocked
from pypanther.rules.cisco_umbrella_dns.fuzzy_matching_domains import (
    CiscoUmbrellaDNSFuzzyMatching as CiscoUmbrellaDNSFuzzyMatching,
)
from pypanther.rules.cisco_umbrella_dns.suspicious_domains import (
    CiscoUmbrellaDNSSuspicious as CiscoUmbrellaDNSSuspicious,
)
