from pypanther.rules.zendesk.zendesk_mobile_app_access import (
    ZendeskMobileAppAccessUpdated as ZendeskMobileAppAccessUpdated,
)
from pypanther.rules.zendesk.zendesk_new_api_token import ZendeskNewAPIToken as ZendeskNewAPIToken
from pypanther.rules.zendesk.zendesk_new_owner import ZendeskAccountOwnerChanged as ZendeskAccountOwnerChanged
from pypanther.rules.zendesk.zendesk_sensitive_data_redaction import (
    ZendeskSensitiveDataRedactionOff as ZendeskSensitiveDataRedactionOff,
)
from pypanther.rules.zendesk.zendesk_user_assumption import ZendeskUserAssumption as ZendeskUserAssumption
from pypanther.rules.zendesk.zendesk_user_role import ZendeskUserRoleChanged as ZendeskUserRoleChanged
from pypanther.rules.zendesk.zendesk_user_suspension import ZendeskUserSuspension as ZendeskUserSuspension
