mutation CreateOrUpdateSchema($input: CreateOrUpdateSchemaInput!) {
    createOrUpdateSchema(input: $input) {
        schema {
            createdAt
            description
            isManaged
            name
            referenceURL
            revision
            spec
            updatedAt
        }
    }
}