from pypanther.rules.gitlab.gitlab_audit_password_reset_multiple_emails import (
    GitLabAuditPasswordResetMultipleEmails as GitLabAuditPasswordResetMultipleEmails,
)
from pypanther.rules.gitlab.gitlab_production_password_reset_multiple_emails import (
    GitLabProductionPasswordResetMultipleEmails as GitLabProductionPasswordResetMultipleEmails,
)
